/* ------------------------------------------------------------------------- *
 * Copyright 2002-2024, OpenNebula Project, OpenNebula Systems               *
 *                                                                           *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may   *
 * not use this file except in compliance with the License. You may obtain   *
 * a copy of the License at                                                  *
 *                                                                           *
 * http://www.apache.org/licenses/LICENSE-2.0                                *
 *                                                                           *
 * Unless required by applicable law or agreed to in writing, software       *
 * distributed under the License is distributed on an "AS IS" BASIS,         *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  *
 * See the License for the specific language governing permissions and       *
 * limitations under the License.                                            *
 * ------------------------------------------------------------------------- */
import { createForm } from 'client/utils'
import {
  SCHEMA,
  FIELDS,
} from 'client/components/Forms/Group/EditAdminsForm/schema'

const EditAdminsForm = createForm(SCHEMA, FIELDS, {
  transformInitialValue: (admins, schema) => ({
    ...schema.cast({ admins }, { stripUnknown: false }),
  }),
  transformBeforeSubmit: (formData, initialValues) => {
    const adminsToAdd = formData?.admins.filter(
      (id) => !initialValues.includes(id)
    )
    const adminsToRemove = initialValues.filter(
      (id) => id && !formData?.admins.includes(id)
    )

    return { adminsToAdd, adminsToRemove }
  },
})

export default EditAdminsForm
